{------------------------------------------------------------------------------
   _____  ____  ____  ____
  (  _  )(_  _)(_  _)( ___) Object-oriented
   )(_)(   )(    )(   )__)  Tengwar
  (_____) (__)  (__) (____) Transcription
  version 0.1.1             Engine

  by Michal Nowakowski </EMAIL_NAME/@/EMAIL_DOMAIN/>

  OTTE is an ObjectPascal unit that provides you with an easy way to
  create a transcriber (not translator!) from Latin alphabet to Tolkien's
  Tengwar and back, like famous TengScribe or YaTT, in Delphi or FPC.

  Although OTTE's transcription back from Tengwar to Latin could be much
  better for modes that place tehtar over the following consonant (like
  English or Sindarin Classic), and does not support Narmacil's TMF modes,
  OTTE is much faster even than Narmacil and still in development,
  so there may be new things added and old improved.

  This is a Narmacil-style shared library for programs in any language.
  Due to OTTE's incapability of loading Narmacil's TMF mode files,
  LoadMode will reject them, and SetSmart() won't do anything and
  GetSmart() will give deceiving results (exactly what You passed to
  SetSmart()). These inconveniences are to be solved before final
  release of OTTE.
                                 --*--

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

------------------------------------------------------------------------------}
library tengwar;
{$I otte_cfg.pas}

uses
  {$IFDEF DELPHI}windows,{$ENDIF}
  sysutils, //ExtractFileName/Path, delphi also strpas
  otte;

type
  bool = ByteBool;

var
  {$IFDEF DELPHI}
    SavedDllProc: procedure(Reason: integer);
    Attached: integer;
  {$ELSE}
    SavedExitProc: procedure;
  {$ENDIF}  

  engie: TotteTranscriber;
  alert, SmartBackTranscription: bool;
  ModeName, ModeDescription, ModePath, TranRes: ansistring;

//----------------------------------------------------------------------------
function Roman2Tengwar(s: pchar): pchar;
{$IFDEF WINDOWS}stdcall{$ELSE}cdecl{$ENDIF};
begin
  TranRes := engie.TranscribeLine(false, strpas(s));
  Result := @(TranRes[1]);
end;

//----------------------------------------------------------------------------
function Tengwar2Roman(s: pchar): pchar;
{$IFDEF WINDOWS}stdcall{$ELSE}cdecl{$ENDIF};
begin
  TranRes := engie.TranscribeLine(true, strpas(s));
  Result := @(TranRes[1]);
end;

//----------------------------------------------------------------------------
procedure LoadMode(Path: pchar);
{$IFDEF WINDOWS}stdcall{$ELSE}cdecl{$ENDIF};
var s: ansistring;
begin
  s := strpas(Path);
  engie.LoadFromFile(s);
  ModeName := ExtractFileName(s);
  ModePath := ExtractFilePath(s);
  ModeDescription := engie.description;
end;

//----------------------------------------------------------------------------
function GetLSD: bool;
{$IFDEF WINDOWS}stdcall{$ELSE}cdecl{$ENDIF};
begin
  result := engie.CirNum;
end;

//----------------------------------------------------------------------------
procedure SetLSD(value: bool);
{$IFDEF WINDOWS}stdcall{$ELSE}cdecl{$ENDIF};
begin
  engie.CirNum := value;
end;

//----------------------------------------------------------------------------
function GetDecimal: bool;
{$IFDEF WINDOWS}stdcall{$ELSE}cdecl{$ENDIF};
begin
  result := not engie.B12Num;
end;

//----------------------------------------------------------------------------
procedure SetDecimal(value: bool);
{$IFDEF WINDOWS}stdcall{$ELSE}cdecl{$ENDIF};
begin
  engie.B12Num := not value;
end;

//----------------------------------------------------------------------------
function GetDigits: bool;
{$IFDEF WINDOWS}stdcall{$ELSE}cdecl{$ENDIF};
begin
  result := engie.CirNum;
end;

//----------------------------------------------------------------------------
procedure SetDigits(value: bool);
{$IFDEF WINDOWS}stdcall{$ELSE}cdecl{$ENDIF};
begin
  engie.DotNum := value;
end;

//----------------------------------------------------------------------------
function GetAlert: bool;
{$IFDEF WINDOWS}stdcall{$ELSE}cdecl{$ENDIF};
begin
  result := alert;
end;

//----------------------------------------------------------------------------
procedure SetAlert(value: bool);
{$IFDEF WINDOWS}stdcall{$ELSE}cdecl{$ENDIF};
begin
  alert := value;
end;

//----------------------------------------------------------------------------
function GetModePath: pchar;
{$IFDEF WINDOWS}stdcall{$ELSE}cdecl{$ENDIF};
begin
  result := @(ModePath[1]);
end;

//----------------------------------------------------------------------------
function GetModeName: pchar;
{$IFDEF WINDOWS}stdcall{$ELSE}cdecl{$ENDIF};
begin
  result := @(ModeName[1]);
end;

//----------------------------------------------------------------------------
function GetModeComment: pchar;
{$IFDEF WINDOWS}stdcall{$ELSE}cdecl{$ENDIF};
begin
  result := @(ModeDescription[1]);
end;

//----------------------------------------------------------------------------
function GetModeCaseSensitivity: bool;
{$IFDEF WINDOWS}stdcall{$ELSE}cdecl{$ENDIF};
begin
  result := engie.CaseSensitive;
end;

//----------------------------------------------------------------------------
function GetSmart: bool;
{$IFDEF WINDOWS}stdcall{$ELSE}cdecl{$ENDIF};
begin
  result := SmartBackTranscription;
end;

//----------------------------------------------------------------------------
procedure SetSmart(value: bool);
{$IFDEF WINDOWS}stdcall{$ELSE}cdecl{$ENDIF};
begin
  SmartBackTranscription := value;
end;

//----------------------------------------------------------------------------
function GetSeparateNums: bool;
{$IFDEF WINDOWS}stdcall{$ELSE}cdecl{$ENDIF};
begin
  result := engie.SepNum;
end;

//----------------------------------------------------------------------------
procedure SetSeparateNums(value: bool);
{$IFDEF WINDOWS}stdcall{$ELSE}cdecl{$ENDIF};
begin
  engie.SepNum := value;
end;

//----------------------------------------------------------------------------
exports
  Roman2Tengwar,
  Tengwar2Roman,
  LoadMode,
  GetLSD,
  SetLSD,
  GetDecimal,
  SetDecimal,
  GetDigits,
  SetDigits,
  GetAlert,
  SetAlert,
  GetModePath,
  GetModeName,
  GetModeComment,
  GetModeCaseSensitivity,
  GetSmart,
  SetSmart,
  GetSeparateNums,
  SetSeparateNums;

//----------------------------------------------------------------------------
procedure InitMyDllVars;
begin
  alert := false;
  SmartBackTranscription := false;
  ModeName := '';
  ModePath := '';
  ModeDescription := '';
end;

//----------------------------------------------------------------------------
{$IFDEF DELPHI}
procedure MyDllProc(Reason: integer);
begin
  case Reason of
    DLL_PROCESS_DETACH: Dec(Attached);
    DLL_PROCESS_ATTACH: inc(Attached);
    DLL_THREAD_DETACH: dec(attached);
    DLL_THREAD_ATTACH: inc(attached);
  end;

  if attached = 0 then begin
    if assigned(engie) then engie.destroy;
    InitMyDllVars;
  end
  else if not assigned(engie) then begin
    engie := TotteTranscriber.create;
    InitMyDllVars;
  end;

  if assigned(SavedDllProc) then
    SavedDllProc(Reason);
end;

{$ELSE}
procedure MyExitProc;
begin
  if assigned(engie) then engie.destroy;
  if assigned(SavedExitProc) then SavedExitProc();
end;

{$ENDIF}
//----------------------------------------------------------------------------

begin
  {$IFDEF DELPHI}
    Attached := 1;
    SavedDllProc := DllProc;
    DllProc := @MyDllProc;
  {$ELSE}
    SavedExitProc := ExitProc;
    ExitProc := @MyExitProc;
  {$ENDIF}

  engie := TotteTranscriber.create;
  InitMyDllVars;
end.

